@@ -1,8 +1,8 @@
 #
 # This file is part of pysmi software.
 #
-# Copyright (c) 2015-2017, Ilya Etingof <etingof@gmail.com>
-# License: http://pysmi.sf.net/license.html
+# Copyright (c) 2015-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysmi/license.html
 #
 import sys
 import re
@@ -72,6 +72,7 @@ class JsonCodeGen(AbstractCodeGen):
         self._importMap = {}
         self._out = {}  # k, v = name, generated code
         self._moduleIdentityOid = None
+        self._moduleRevision = None
         self._enterpriseOid = None
         self._oids = set()
         self._complianceOids = []
@@ -273,6 +274,8 @@ class JsonCodeGen(AbstractCodeGen):
         if revisions:
             outDict['revisions'] = revisions
 
+            self._moduleRevision = revisions[0]['revision']
+
         if self.genRules['text']:
             if lastUpdated:
                 outDict['lastupdated'] = lastUpdated
@@ -633,7 +636,7 @@ class JsonCodeGen(AbstractCodeGen):
 
         else:  # symbol (oid as defval) or name for enumeration member
             if (defvalType[0][0] == 'ObjectIdentifier' and
-                (defval in self.symbolTable[self.moduleName[0]] or defval in self._importMap)):  # oid
+                    (defval in self.symbolTable[self.moduleName[0]] or defval in self._importMap)):  # oid
 
                 module = self._importMap.get(defval, self.moduleName[0])
 
@@ -645,9 +648,13 @@ class JsonCodeGen(AbstractCodeGen):
                     raise error.PySmiSemanticError('no symbol "%s" in module "%s"' % (defval, module))
 
             # enumeration
-            elif (defvalType[0][0] in ('Integer32', 'Integer') and
-                  isinstance(defvalType[1], list) and defval in dict(defvalType[1])):
-                outDict.update(value=defval, format='enum')
+            elif defvalType[0][0] in ('Integer32', 'Integer') and isinstance(defvalType[1], list):
+                if isinstance(defval, list):  # buggy MIB: DEFVAL { { ... } }
+                    defval = [dv for dv in defval if dv in dict(defvalType[1])]
+                    if defval:
+                        outDict.update(value=defval[0], format='enum')
+                elif defval in dict(defvalType[1]):  # good MIB: DEFVAL { ... }
+                    outDict.update(value=defval, format='enum')
 
             elif defvalType[0][0] == 'Bits':
                 defvalBits = []
@@ -683,6 +690,9 @@ class JsonCodeGen(AbstractCodeGen):
     def genStatus(self, data):
         return data[0]
 
+    def genProductRelease(self, data):
+        return data[0]
+
     def genEnumSpec(self, data):
         items = data[0]
         return {'enumeration': dict(items)}
@@ -886,7 +896,7 @@ class JsonCodeGen(AbstractCodeGen):
         'trapTypeClause': genTrapType,
         'typeDeclaration': genTypeDeclaration,
         'valueDeclaration': genValueDeclaration,
-
+        'PRODUCT-RELEASE': genProductRelease,
         'ApplicationSyntax': genSimpleSyntax,
         'BitNames': genBitNames,
         'BITS': genBits,
@@ -957,6 +967,7 @@ class JsonCodeGen(AbstractCodeGen):
         return MibInfo(oid=moduleOid,
                        identity=self._moduleIdentityOid,
                        name=self.moduleName[0],
+                       revision=self._moduleRevision,
                        oids=self._oids,
                        enterprise=self._enterpriseOid,
                        compliance=self._complianceOids,
