@@ -1,8 +1,8 @@
 #
 # This file is part of pysmi software.
 #
-# Copyright (c) 2015-2017, Ilya Etingof <etingof@gmail.com>
-# License: http://pysmi.sf.net/license.html
+# Copyright (c) 2015-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysmi/license.html
 #
 import sys
 import re
@@ -116,6 +116,7 @@ class PySnmpCodeGen(AbstractCodeGen):
         self._importMap = {}
         self._out = {}  # k, v = name, generated code
         self._moduleIdentityOid = None
+        self._moduleRevision = None
         self.moduleName = ['DUMMY']
         self.genRules = {'text': True}
         self.symbolTable = {}
@@ -340,7 +341,9 @@ if getattr(mibBuilder, 'version', (0, 0, 0)) > (4, 4, 0):
         outStr = name + ' = ModuleIdentity(' + oidStr + ')' + label + '\n'
 
         if revisionsAndDescrs:
-            revisions, descriptions = revisionsAndDescrs
+            last_revision, revisions, descriptions = revisionsAndDescrs
+
+            self._moduleRevision = last_revision
 
             if revisions:
                 outStr += name + revisions + '\n'
@@ -826,10 +829,14 @@ for _%(name)s_obj in [%(objects)s]:
                     raise error.PySmiSemanticError('no symbol "%s" in module "%s"' % (defval, module))
 
             # enumeration
-            elif (defvalType[0][0] in ('Integer32', 'Integer') and
-                    isinstance(defvalType[1], list) and
-                    defval in dict(defvalType[1])):
-                val = dorepr(defval)
+            elif defvalType[0][0] in ('Integer32', 'Integer') and isinstance(defvalType[1], list):
+                if isinstance(defval, list):  # buggy MIB: DEFVAL { { ... } }
+                    defval = [dv for dv in defval if dv in dict(defvalType[1])]
+                    val = defval and dorepr(defval[0]) or ''
+                elif defval in dict(defvalType[1]):  # good MIB: DEFVAL { ... }
+                    val = dorepr(defval)
+                else:
+                    val = ''
 
             elif defvalType[0][0] == 'Bits':
                 defvalBits = []
@@ -1030,7 +1037,9 @@ for _%(name)s_obj in [%(objects)s]:
             [dorepr(self.textFilter('description', x[1][1])) for x in data[0]]
         )
 
-        return revisions, descriptions
+        lastRevision = data[0][0][0]
+
+        return lastRevision, revisions, descriptions
 
     def genRow(self, data, classmode=False):
         row = data[0]
@@ -1168,7 +1177,7 @@ for _%(name)s_obj in [%(objects)s]:
 
         if 'comments' in kwargs:
             out = ''.join(['# %s\n' % x for x in kwargs['comments']]) + '#\n' + out
-            out = '#\n# PySNMP MIB module %s (http://pysnmp.sf.net)\n' % self.moduleName[0] + out
+            out = '#\n# PySNMP MIB module %s (http://snmplabs.com/pysmi)\n' % self.moduleName[0] + out
 
         debug.logger & debug.flagCodegen and debug.logger(
             'canonical MIB name %s (%s), imported MIB(s) %s, Python code size %s bytes' % (
@@ -1177,6 +1186,7 @@ for _%(name)s_obj in [%(objects)s]:
         return MibInfo(oid=moduleOid,
                        identity=self._moduleIdentityOid,
                        name=self.moduleName[0],
+                       revision=self._moduleRevision,
                        oids=[],
                        enterprise=None,
                        compliance=[],
@@ -1194,7 +1204,7 @@ for _%(name)s_obj in [%(objects)s]:
 
         if 'comments' in kwargs:
             out = ''.join(['# %s\n' % x for x in kwargs['comments']]) + '#\n' + out
-            out = '#\n# PySNMP MIB indices (http://pysnmp.sf.net)\n' + out
+            out = '#\n# PySNMP MIB indices (http://snmplabs.com/pysmi)\n' + out
 
         debug.logger & debug.flagCodegen and debug.logger(
             'OID->MIB index built, %s entries, %s bytes' % (count, len(out)))
