@@ -1,8 +1,8 @@
 #
 # This file is part of pysmi software.
 #
-# Copyright (c) 2015-2017, Ilya Etingof <etingof@gmail.com>
-# License: http://pysmi.sf.net/license.html
+# Copyright (c) 2015-2018, Ilya Etingof <etingof@gmail.com>
+# License: http://snmplabs.com/pysmi/license.html
 #
 import sys
 import os
@@ -156,6 +156,22 @@ class MibCompiler(object):
 
         return self
 
+    def _get_system_info(self):
+
+        try:
+            platform_info = os.uname()
+
+        except AttributeError:
+            platform_info = ('?',) * 6
+
+        try:
+            user_info = getpwuid(os.getuid())
+
+        except Exception:
+            user_info = ('?',) * 7
+
+        return platform_info, user_info
+
     def compile(self, *mibnames, **options):
         """Transform requested and possibly referred MIBs.
 
@@ -191,6 +207,7 @@ class MibCompiler(object):
         builtMibs = {}
         symbolTableMap = {}
         mibsToParse = [x for x in mibnames]
+        canonicalMibNames = {}
 
         while mibsToParse:
             mibname = mibsToParse.pop(0)
@@ -223,6 +240,11 @@ class MibCompiler(object):
 
                         mibsToParse.extend(mibInfo.imported)
 
+                        if fileInfo.name in mibnames:
+                            if mibInfo.name not in canonicalMibNames:
+                                canonicalMibNames[mibInfo.name] = []
+                            canonicalMibNames[mibInfo.name].append(fileInfo.name)
+
                         debug.logger & debug.flagCompiler and debug.logger(
                             '%s (%s) read from %s, immediate dependencies: %s' % (
                                 mibInfo.name, mibname, fileInfo.path, ', '.join(mibInfo.imported) or '<none>'))
@@ -258,7 +280,7 @@ class MibCompiler(object):
                     processed[mibname] = statusMissing
 
         debug.logger & debug.flagCompiler and debug.logger(
-            'MIBs analized %s, MIBs failed %s' % (len(parsedMibs), len(failedMibs)))
+            'MIBs analyzed %s, MIBs failed %s' % (len(parsedMibs), len(failedMibs)))
 
         #
         # See what MIBs need generating
@@ -297,7 +319,7 @@ class MibCompiler(object):
                 debug.logger & debug.flagCompiler and debug.logger(
                     'no suitable compiled MIB %s found anywhere' % mibname)
 
-                if options.get('noDeps') and mibname not in mibnames:
+                if options.get('noDeps') and mibname not in canonicalMibNames:
                     debug.logger & debug.flagCompiler and debug.logger(
                         'excluding imported MIB %s from code generation' % mibname)
                     del parsedMibs[mibname]
@@ -316,13 +338,13 @@ class MibCompiler(object):
 
             debug.logger & debug.flagCompiler and debug.logger('compiling %s read from %s' % (mibname, fileInfo.path))
 
+            platform_info, user_info = self._get_system_info()
+
             comments = [
                 'ASN.1 source %s' % fileInfo.path,
                 'Produced by %s-%s at %s' % (packageName, packageVersion, time.asctime()),
-                'On host %s platform %s version %s by user %s' % (
-                    hasattr(os, 'uname') and os.uname()[1] or '?', hasattr(os, 'uname') and os.uname()[0] or '?',
-                    hasattr(os, 'uname') and os.uname()[2] or '?',
-                    hasattr(os, 'getuid') and getpwuid(os.getuid())[0] or '?'),
+                'On host %s platform %s version %s by user %s' % (platform_info[1], platform_info[0],
+                                                                  platform_info[2], user_info[0]),
                 'Using Python version %s' % sys.version.split('\n')[0]
             ]
 
@@ -362,7 +384,7 @@ class MibCompiler(object):
         #
 
         for mibname in failedMibs.copy():
-            if options.get('noDeps') and mibname not in mibnames:
+            if options.get('noDeps') and mibname not in canonicalMibNames:
                 debug.logger & debug.flagCompiler and debug.logger('excluding imported MIB %s from borrowing' % mibname)
                 continue
 
@@ -425,7 +447,7 @@ class MibCompiler(object):
                 debug.logger & debug.flagCompiler and debug.logger(
                     'no suitable compiled MIB %s found anywhere' % mibname)
 
-                if options.get('noDeps') and mibname not in mibnames:
+                if options.get('noDeps') and mibname not in canonicalMibNames:
                     debug.logger & debug.flagCompiler and debug.logger(
                         'excluding imported MIB %s from borrowing' % mibname)
                     processed[mibname] = statusUntouched
@@ -484,6 +506,7 @@ class MibCompiler(object):
                         oid=mibInfo.oid,
                         oids=mibInfo.oids,
                         identity=mibInfo.identity,
+                        revision=mibInfo.revision,
                         enterprise=mibInfo.enterprise,
                         compliance=mibInfo.compliance,
                     )
@@ -506,13 +529,15 @@ class MibCompiler(object):
         return processed
 
     def buildIndex(self, processedMibs, **options):
+        platform_info, user_info = self._get_system_info()
+
         comments = [
             'Produced by %s-%s at %s' % (packageName, packageVersion, time.asctime()),
-            'On host %s platform %s version %s by user %s' % (
-                hasattr(os, 'uname') and os.uname()[1] or '?', hasattr(os, 'uname') and os.uname()[0] or '?',
-                hasattr(os, 'uname') and os.uname()[2] or '?', hasattr(os, 'getuid') and getpwuid(os.getuid())[0]) or '?',
+            'On host %s platform %s version %s by user %s' % (platform_info[1], platform_info[0],
+                                                              platform_info[2], user_info[0]),
             'Using Python version %s' % sys.version.split('\n')[0]
         ]
+
         try:
             self._writer.putData(
                 self.indexFile,
